//////////////////////////////////////////////////////////
// Sketch library for ADT7410 (Analog Devices, Inc.)
// Programmed by KEI SAKAKI. http://kei-sakaki.jp/

#include "adt7410.h"
#include "Wire.h"

ADT7410::ADT7410(int i2c_address) {
  Wire.begin();
  this->i2c_address = i2c_address;
  softwareReset();
}

ADT7410::~ADT7410() {
}

int ADT7410::readTemperatureRaw(bool width16) {
  int raw;
  
  Wire.beginTransmission(i2c_address);
  Wire.write(0x03);                  // 0x03 : Configuration Register
  Wire.write(width16 ? 0x80 : 0x00);  // bit7 : Resolution : 0 = 13-bit resolution / 1 = 16-bit resolution. (see Data Sheet Rev.A Page 13)
  Wire.endTransmission();

  Wire.requestFrom(i2c_address, 2);
  raw = (Wire.read()) & 0xff;
  raw <<= 8;
  raw |= (Wire.read()) & 0xff;  // high byte order

  if(!width16) {
    raw >>= 3;  // 13-bit mode : bit 0-2 : flags
  }

  return raw;
}

float ADT7410::readTemperature(bool width16) {
  float temp;
  int raw = readTemperatureRaw(width16);
  
  // (see Data Sheet Rev.A Page 12)
  if(width16) {
    if(raw & 0x8000) {
      raw =- 65536;
    }
    temp = ((float) raw) / 128.0f;
  } else {
    if(raw & 0x1000) {
      raw =- 8192;
    }
    temp = ((float) raw) / 16.0f;
  }
  
  return temp;
}

void ADT7410::softwareReset() {
  Wire.beginTransmission(i2c_address);
  Wire.write(0x2f);  // 0x2f : Software reset
  Wire.endTransmission();
  delayMicroseconds(200);  // ADT7410 need 200 us after reset (see Data Sheet Rev.A Page 19)
}

void ADT7410::shutdown() {
  Wire.beginTransmission(i2c_address);
  Wire.write(0x03);  // 0x03 : Configuration Register
  Wire.write(0x60);  // bit5-6 : 00 = Continuous / 01 = One shot / 10 = 1 SPS mode / 11 = shutdown(Power down) (see Data Sheet Rev.A Page 14)
  Wire.endTransmission();
}
